@@ -2436,12 +2436,32 @@ static int vfswrap_fchmod(vfs_handle_struct *handle, files_struct *fsp, mode_t m
 	int result;
 
 	START_PROFILE(syscall_fchmod);
-#if defined(HAVE_FCHMOD)
-	result = fchmod(fsp_get_io_fd(fsp), mode);
-#else
-	result = -1;
-	errno = ENOSYS;
-#endif
+
+	if (!fsp->fsp_flags.is_pathref) {
+		result = fchmod(fsp_get_io_fd(fsp), mode);
+		END_PROFILE(syscall_fchmod);
+		return result;
+	}
+
+	if (fsp->fsp_flags.have_proc_fds) {
+		int fd = fsp_get_pathref_fd(fsp);
+		const char *p = NULL;
+		char buf[PATH_MAX];
+
+		p = sys_proc_fd_path(fd, buf, sizeof(buf));
+		if (p != NULL) {
+			result = chmod(p, mode);
+		} else {
+			result = -1;
+		}
+		END_PROFILE(syscall_fchmod);
+		return result;
+	}
+
+	/*
+	 * This is no longer a handle based call.
+	 */
+	result = chmod(fsp->fsp_name->base_name, mode);
 
 	END_PROFILE(syscall_fchmod);
 	return result;
@@ -2453,7 +2473,31 @@ static int vfswrap_fchown(vfs_handle_struct *handle, files_struct *fsp, uid_t ui
 	int result;
 
 	START_PROFILE(syscall_fchown);
-	result = fchown(fsp_get_io_fd(fsp), uid, gid);
+	if (!fsp->fsp_flags.is_pathref) {
+		result = fchown(fsp_get_io_fd(fsp), uid, gid);
+		END_PROFILE(syscall_fchown);
+		return result;
+	}
+
+	if (fsp->fsp_flags.have_proc_fds) {
+		int fd = fsp_get_pathref_fd(fsp);
+		const char *p = NULL;
+		char buf[PATH_MAX];
+
+		p = sys_proc_fd_path(fd, buf, sizeof(buf));
+		if (p != NULL) {
+			result = chown(p, uid, gid);
+		} else {
+			result = -1;
+		}
+		END_PROFILE(syscall_fchown);
+		return result;
+	}
+
+	/*
+	 * This is no longer a handle based call.
+	 */
+	result = chown(fsp->fsp_name->base_name, uid, gid);
 	END_PROFILE(syscall_fchown);
 	return result;
 #else
