@@ -27,6 +27,7 @@
 #include "smbd/smbd.h"
 #include "lib/param/loadparm.h"
 #include "lib/util/tevent_ntstatus.h"
+#include "lib/util/string_wrappers.h"
 
 static NTSTATUS get_file_handle_for_metadata(connection_struct *conn,
 				const struct smb_filename *smb_fname,
@@ -84,17 +85,6 @@ static uint32_t filter_mode_by_protocol(uint32_t mode)
 	return mode;
 }
 
-static int set_link_read_only_flag(const SMB_STRUCT_STAT *const sbuf)
-{
-#ifdef S_ISLNK
-#if LINKS_READ_ONLY
-	if (S_ISLNK(sbuf->st_mode) && S_ISDIR(sbuf->st_mode))
-		return FILE_ATTRIBUTE_READONLY;
-#endif
-#endif
-	return 0;
-}
-
 /****************************************************************************
  Change a dos mode to a unix mode.
     Base permission for files:
@@ -120,7 +110,7 @@ static int set_link_read_only_flag(const SMB_STRUCT_STAT *const sbuf)
 
 mode_t unix_mode(connection_struct *conn, int dosmode,
 		 const struct smb_filename *smb_fname,
-		 const char *inherit_from_dir)
+		 struct smb_filename *smb_fname_parent)
 {
 	mode_t result = (S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IWGRP | S_IWOTH);
 	mode_t dir_mode = 0; /* Mode of the inherit_from directory if
@@ -130,30 +120,15 @@ mode_t unix_mode(connection_struct *conn, int dosmode,
 		result &= ~(S_IWUSR | S_IWGRP | S_IWOTH);
 	}
 
-	if ((inherit_from_dir != NULL) && lp_inherit_permissions(SNUM(conn))) {
-		struct smb_filename *smb_fname_parent;
-
-		DEBUG(2, ("unix_mode(%s) inheriting from %s\n",
+	if ((smb_fname_parent != NULL) && lp_inherit_permissions(SNUM(conn))) {
+		DBG_DEBUG("[%s] inheriting from [%s]\n",
 			  smb_fname_str_dbg(smb_fname),
-			  inherit_from_dir));
-
-		smb_fname_parent = synthetic_smb_fname(talloc_tos(),
-					inherit_from_dir,
-					NULL,
-					NULL,
-					smb_fname->flags);
-		if (smb_fname_parent == NULL) {
-			DEBUG(1,("unix_mode(%s) failed, [dir %s]: No memory\n",
-				 smb_fname_str_dbg(smb_fname),
-				 inherit_from_dir));
-			return(0);
-		}
+			  smb_fname_str_dbg(smb_fname_parent));
 
 		if (SMB_VFS_STAT(conn, smb_fname_parent) != 0) {
-			DEBUG(4,("unix_mode(%s) failed, [dir %s]: %s\n",
-				 smb_fname_str_dbg(smb_fname),
-				 inherit_from_dir, strerror(errno)));
-			TALLOC_FREE(smb_fname_parent);
+			DBG_ERR("stat failed [%s]: %s\n",
+				smb_fname_str_dbg(smb_fname_parent),
+				strerror(errno));
 			return(0);      /* *** shouldn't happen! *** */
 		}
 
@@ -163,7 +138,6 @@ mode_t unix_mode(connection_struct *conn, int dosmode,
 			 smb_fname_str_dbg(smb_fname), (int)dir_mode));
 		/* Clear "result" */
 		result = 0;
-		TALLOC_FREE(smb_fname_parent);
 	} 
 
 	if (IS_DOS_DIR(dosmode)) {
@@ -233,7 +207,10 @@ static uint32_t dos_mode_from_sbuf(connection_struct *conn,
 		}
 	} else if (ro_opts == MAP_READONLY_PERMISSIONS) {
 		/* Check actual permissions for read-only. */
-		if (!can_write_to_file(conn, smb_fname)) {
+		if (!can_write_to_file(conn,
+				conn->cwd_fsp,
+				smb_fname))
+		{
 			result |= FILE_ATTRIBUTE_READONLY;
 		}
 	} /* Else never set the readonly bit. */
@@ -250,8 +227,6 @@ static uint32_t dos_mode_from_sbuf(connection_struct *conn,
 	if (S_ISDIR(smb_fname->st.st_ex_mode))
 		result = FILE_ATTRIBUTE_DIRECTORY | (result & FILE_ATTRIBUTE_READONLY);
 
-	result |= set_link_read_only_flag(&smb_fname->st);
-
 	dos_mode_debug_print(__func__, result);
 
 	return result;
@@ -388,8 +363,7 @@ NTSTATUS parse_dos_attribute_blob(struct smb_filename *smb_fname,
 	return NT_STATUS_OK;
 }
 
-NTSTATUS get_ea_dos_attribute(connection_struct *conn,
-			      struct smb_filename *smb_fname,
+NTSTATUS fget_ea_dos_attribute(struct files_struct *fsp,
 			      uint32_t *pattr)
 {
 	DATA_BLOB blob;
@@ -397,63 +371,28 @@ NTSTATUS get_ea_dos_attribute(connection_struct *conn,
 	fstring attrstr;
 	NTSTATUS status;
 
-	if (!lp_store_dos_attributes(SNUM(conn))) {
+	if (!lp_store_dos_attributes(SNUM(fsp->conn))) {
 		return NT_STATUS_NOT_IMPLEMENTED;
 	}
 
 	/* Don't reset pattr to zero as we may already have filename-based attributes we
 	   need to preserve. */
 
-	sizeret = SMB_VFS_GETXATTR(conn, smb_fname,
-				   SAMBA_XATTR_DOS_ATTRIB, attrstr,
-				   sizeof(attrstr));
-	if (sizeret == -1 && errno == EACCES) {
-		int saved_errno = 0;
-
-		/*
-		 * According to MS-FSA 2.1.5.1.2.1 "Algorithm to Check Access to
-		 * an Existing File" FILE_LIST_DIRECTORY on a directory implies
-		 * FILE_READ_ATTRIBUTES for directory entries. Being able to
-		 * stat() a file implies FILE_LIST_DIRECTORY for the directory
-		 * containing the file.
-		 */
-
-		if (!VALID_STAT(smb_fname->st)) {
-			/*
-			 * Safety net: dos_mode() already checks this, but as we
-			 * become root based on this, add an additional layer of
-			 * defense.
-			 */
-			DBG_ERR("Rejecting root override, invalid stat [%s]\n",
-				smb_fname_str_dbg(smb_fname));
-			return NT_STATUS_ACCESS_DENIED;
-		}
-
-		become_root();
-		sizeret = SMB_VFS_GETXATTR(conn, smb_fname,
-					   SAMBA_XATTR_DOS_ATTRIB,
-					   attrstr,
-					   sizeof(attrstr));
-		if (sizeret == -1) {
-			saved_errno = errno;
-		}
-		unbecome_root();
-
-		if (saved_errno != 0) {
-			errno = saved_errno;
-		}
-	}
+	sizeret = SMB_VFS_FGETXATTR(fsp->base_fsp ? fsp->base_fsp : fsp,
+				    SAMBA_XATTR_DOS_ATTRIB,
+				    attrstr,
+				    sizeof(attrstr));
 	if (sizeret == -1) {
 		DBG_INFO("Cannot get attribute "
 			 "from EA on file %s: Error = %s\n",
-			 smb_fname_str_dbg(smb_fname), strerror(errno));
+			 fsp_str_dbg(fsp), strerror(errno));
 		return map_nt_error_from_unix(errno);
 	}
 
 	blob.data = (uint8_t *)attrstr;
 	blob.length = sizeret;
 
-	status = parse_dos_attribute_blob(smb_fname, blob, pattr);
+	status = parse_dos_attribute_blob(fsp->fsp_name, blob, pattr);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
 	}
@@ -546,14 +485,19 @@ NTSTATUS set_ea_dos_attribute(connection_struct *conn,
 			return NT_STATUS_ACCESS_DENIED;
 		}
 
-		status = smbd_check_access_rights(conn, smb_fname, false,
-						  FILE_WRITE_ATTRIBUTES);
+		status = smbd_check_access_rights(conn,
+					conn->cwd_fsp,
+					smb_fname,
+					false,
+					FILE_WRITE_ATTRIBUTES);
 		if (NT_STATUS_IS_OK(status)) {
 			set_dosmode_ok = true;
 		}
 
 		if (!set_dosmode_ok && lp_dos_filemode(SNUM(conn))) {
-			set_dosmode_ok = can_write_to_file(conn, smb_fname);
+			set_dosmode_ok = can_write_to_file(conn,
+						conn->cwd_fsp,
+						smb_fname);
 		}
 
 		if (!set_dosmode_ok) {
@@ -653,6 +597,7 @@ uint32_t dos_mode_msdfs(connection_struct *conn,
  * check whether a file or directory is flagged as compressed.
  */
 static NTSTATUS dos_mode_check_compressed(connection_struct *conn,
+					  struct files_struct *fsp,
 					  struct smb_filename *smb_fname,
 					  bool *is_compressed)
 {
@@ -664,7 +609,7 @@ static NTSTATUS dos_mode_check_compressed(connection_struct *conn,
 		goto err_out;
 	}
 
-	status = SMB_VFS_GET_COMPRESSION(conn, tmp_ctx, NULL, smb_fname,
+	status = SMB_VFS_FGET_COMPRESSION(conn, tmp_ctx, fsp,
 					 &compression_fmt);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto err_ctx_free;
@@ -719,11 +664,16 @@ static uint32_t dos_mode_from_name(connection_struct *conn,
 
 static uint32_t dos_mode_post(uint32_t dosmode,
 			      connection_struct *conn,
+			      struct files_struct *fsp,
 			      struct smb_filename *smb_fname,
 			      const char *func)
 {
 	NTSTATUS status;
 
+	if (fsp != NULL) {
+		smb_fname = fsp->fsp_name;
+	}
+
 	/*
 	 * According to MS-FSA a stream name does not have
 	 * separate DOS attribute metadata, so we must return
@@ -744,7 +694,7 @@ static uint32_t dos_mode_post(uint32_t dosmode,
 	if (conn->fs_capabilities & FILE_FILE_COMPRESSION) {
 		bool compressed = false;
 
-		status = dos_mode_check_compressed(conn, smb_fname,
+		status = dos_mode_check_compressed(conn, fsp, smb_fname,
 						   &compressed);
 		if (NT_STATUS_IS_OK(status) && compressed) {
 			dosmode |= FILE_ATTRIBUTE_COMPRESSED;
@@ -771,29 +721,43 @@ static uint32_t dos_mode_post(uint32_t dosmode,
  if "store dos attributes" is true.
 ****************************************************************************/
 
-uint32_t dos_mode(connection_struct *conn, struct smb_filename *smb_fname)
+uint32_t fdos_mode(struct files_struct *fsp)
 {
 	uint32_t result = 0;
 	NTSTATUS status = NT_STATUS_OK;
 
-	DEBUG(8,("dos_mode: %s\n", smb_fname_str_dbg(smb_fname)));
+	if (fsp == NULL) {
+		/*
+		 * The pathological case where a callers does
+		 * fdos_mode(smb_fname->fsp) passing a pathref fsp. But as
+		 * smb_fname points at a symlink in POSIX context smb_fname->fsp
+		 * is NULL.
+		 */
+		return FILE_ATTRIBUTE_NORMAL;
+	}
 
-	if (!VALID_STAT(smb_fname->st)) {
+	DBG_DEBUG("%s\n", fsp_str_dbg(fsp));
+
+	if (!VALID_STAT(fsp->fsp_name->st)) {
 		return 0;
 	}
 
+	if (S_ISLNK(fsp->fsp_name->st.st_ex_mode)) {
+		return FILE_ATTRIBUTE_NORMAL;
+	}
+
 	/* Get the DOS attributes via the VFS if we can */
-	status = SMB_VFS_GET_DOS_ATTRIBUTES(conn, smb_fname, &result);
+	status = SMB_VFS_FGET_DOS_ATTRIBUTES(fsp->conn, fsp, &result);
 	if (!NT_STATUS_IS_OK(status)) {
 		/*
 		 * Only fall back to using UNIX modes if we get NOT_IMPLEMENTED.
 		 */
 		if (NT_STATUS_EQUAL(status, NT_STATUS_NOT_IMPLEMENTED)) {
-			result |= dos_mode_from_sbuf(conn, smb_fname);
+			result |= dos_mode_from_sbuf(fsp->conn, fsp->fsp_name);
 		}
 	}
 
-	result = dos_mode_post(result, conn, smb_fname, __func__);
+	result = dos_mode_post(result, fsp->conn, fsp, NULL, __func__);
 	return result;
 }
 
@@ -894,6 +858,7 @@ static void dos_mode_at_vfs_get_dosmode_done(struct tevent_req *subreq)
 	if (NT_STATUS_IS_OK(status)) {
 		state->dosmode = dos_mode_post(state->dosmode,
 					       state->dir_fsp->conn,
+					       NULL,
 					       state->smb_fname,
 					       __func__);
 		tevent_req_done(req);
@@ -916,12 +881,13 @@ static void dos_mode_at_vfs_get_dosmode_done(struct tevent_req *subreq)
 				       path,
 				       NULL,
 				       &state->smb_fname->st,
+				       state->smb_fname->twrp,
 				       0);
 	if (tevent_req_nomem(smb_path, req)) {
 		return;
 	}
 
-	state->dosmode = dos_mode(state->dir_fsp->conn, smb_path);
+	state->dosmode = fdos_mode(state->smb_fname->fsp);
 	tevent_req_done(req);
 	return;
 }
@@ -950,8 +916,11 @@ NTSTATUS dos_mode_at_recv(struct tevent_req *req, uint32_t *dosmode)
  attribute also.
 ********************************************************************/
 
-int file_set_dosmode(connection_struct *conn, struct smb_filename *smb_fname,
-		     uint32_t dosmode, const char *parent_dir, bool newfile)
+int file_set_dosmode(connection_struct *conn,
+		     struct smb_filename *smb_fname,
+		     uint32_t dosmode,
+		     struct smb_filename *parent_dir,
+		     bool newfile)
 {
 	int mask=0;
 	mode_t tmp;
@@ -1080,7 +1049,10 @@ int file_set_dosmode(connection_struct *conn, struct smb_filename *smb_fname,
 		bits on a file. Just like file_ntimes below.
 	*/
 
-	if (!can_write_to_file(conn, smb_fname)) {
+	if (!can_write_to_file(conn,
+			conn->cwd_fsp,
+			smb_fname))
+	{
 		errno = EACCES;
 		return -1;
 	}
@@ -1152,7 +1124,7 @@ NTSTATUS file_set_sparse(connection_struct *conn,
 		return NT_STATUS_ACCESS_DENIED;
 	}
 
-	if (fsp->is_directory) {
+	if (fsp->fsp_flags.is_directory) {
 		DEBUG(9, ("invalid attempt to %s sparse flag on dir %s\n",
 			  (sparse ? "set" : "clear"),
 			  smb_fname_str_dbg(fsp->fsp_name)));
@@ -1177,7 +1149,7 @@ NTSTATUS file_set_sparse(connection_struct *conn,
 		return status;
 	}
 
-	old_dosmode = dos_mode(conn, fsp->fsp_name);
+	old_dosmode = fdos_mode(fsp);
 
 	if (sparse && !(old_dosmode & FILE_ATTRIBUTE_SPARSE)) {
 		new_dosmode = old_dosmode | FILE_ATTRIBUTE_SPARSE;
@@ -1197,7 +1169,7 @@ NTSTATUS file_set_sparse(connection_struct *conn,
 		     FILE_NOTIFY_CHANGE_ATTRIBUTES,
 		     fsp->fsp_name->base_name);
 
-	fsp->is_sparse = sparse;
+	fsp->fsp_flags.is_sparse = sparse;
 
 	return NT_STATUS_OK;
 }
@@ -1253,7 +1225,10 @@ int file_ntimes(connection_struct *conn, const struct smb_filename *smb_fname,
 	 */
 
 	/* Check if we have write access. */
-	if (can_write_to_file(conn, smb_fname)) {
+	if (can_write_to_file(conn,
+			conn->cwd_fsp,
+			smb_fname))
+	{
 		/* We are allowed to become root and change the filetime. */
 		become_root();
 		ret = SMB_VFS_NTIMES(conn, smb_fname, ft);
@@ -1292,7 +1267,7 @@ bool set_sticky_write_time_fsp(struct files_struct *fsp, struct timespec mtime)
 		return true;
 	}
 
-	fsp->write_time_forced = true;
+	fsp->fsp_flags.write_time_forced = true;
 	TALLOC_FREE(fsp->update_write_time_event);
 
 	return set_sticky_write_time_path(fsp->file_id, mtime);
@@ -1318,13 +1293,14 @@ NTSTATUS set_create_timespec_ea(connection_struct *conn,
 					psmb_fname->base_name,
 					NULL,
 					&psmb_fname->st,
+					psmb_fname->twrp,
 					psmb_fname->flags);
 
 	if (smb_fname == NULL) {
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	dosmode = dos_mode(conn, smb_fname);
+	dosmode = fdos_mode(psmb_fname->fsp);
 
 	smb_fname->st.st_ex_btime = create_time;
 
@@ -1386,10 +1362,10 @@ static NTSTATUS get_file_handle_for_metadata(connection_struct *conn,
 
 	file_id = vfs_file_id_from_sbuf(conn, &smb_fname->st);
 
-	for(fsp = file_find_di_first(conn->sconn, file_id);
+	for(fsp = file_find_di_first(conn->sconn, file_id, true);
 			fsp;
-			fsp = file_find_di_next(fsp)) {
-		if (fsp->fh->fd != -1) {
+			fsp = file_find_di_next(fsp, true)) {
+		if (fsp_get_io_fd(fsp) != -1) {
 			*ret_fsp = fsp;
 			return NT_STATUS_OK;
 		}
@@ -1401,11 +1377,16 @@ static NTSTATUS get_file_handle_for_metadata(connection_struct *conn,
 		return NT_STATUS_NO_MEMORY;
 	}
 
+	status = openat_pathref_fsp(conn->cwd_fsp, smb_fname_cp);
+	if (!NT_STATUS_IS_OK(status)) {
+		TALLOC_FREE(smb_fname_cp);
+		return status;
+	}
+
 	/* Opens an INTERNAL_OPEN_ONLY write handle. */
 	status = SMB_VFS_CREATE_FILE(
 		conn,                                   /* conn */
 		NULL,                                   /* req */
-		0,                                      /* root_dir_fid */
 		smb_fname_cp,				/* fname */
 		FILE_WRITE_ATTRIBUTES,			/* access_mask */
 		(FILE_SHARE_READ | FILE_SHARE_WRITE |   /* share_access */
