@@ -33,6 +33,7 @@
 #include "libcli/security/security.h"
 #include "librpc/gen_ndr/ndr_dfsblobs.h"
 #include "lib/tsocket/tsocket.h"
+#include "lib/global_contexts.h"
 
 /**********************************************************************
  Parse a DFS pathname of the form \hostname\service\reqpath
@@ -58,8 +59,7 @@ static NTSTATUS parse_dfs_path(connection_struct *conn,
 				const char *pathname,
 				bool allow_wcards,
 				bool allow_broken_path,
-				struct dfs_path *pdp, /* MUST BE TALLOCED */
-				bool *ppath_contains_wcard)
+				struct dfs_path *pdp) /* MUST BE TALLOCED */
 {
 	const struct loadparm_substitution *lp_sub =
 		loadparm_s3_global_substitution();
@@ -208,20 +208,19 @@ static NTSTATUS parse_dfs_path(connection_struct *conn,
 
   local_path:
 
-	*ppath_contains_wcard = False;
-
 	pdp->reqpath = p;
 
 	/* Rest is reqpath. */
 	if (pdp->posix_path) {
 		status = check_path_syntax_posix(pdp->reqpath);
 	} else {
-		if (allow_wcards) {
-			status = check_path_syntax_wcard(pdp->reqpath,
-					ppath_contains_wcard);
-		} else {
-			status = check_path_syntax(pdp->reqpath);
+		if (!allow_wcards) {
+			bool has_wcard = ms_has_wild(pdp->reqpath);
+			if (has_wcard) {
+				return NT_STATUS_INVALID_PARAMETER;
+			}
 		}
+		status = check_path_syntax(pdp->reqpath);
 	}
 
 	if (!NT_STATUS_IS_OK(status)) {
@@ -243,6 +242,7 @@ static NTSTATUS parse_dfs_path(connection_struct *conn,
 *********************************************************/
 
 static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
+			    struct tevent_context *ev,
 			    struct messaging_context *msg,
 			    connection_struct **pconn,
 			    int snum,
@@ -261,12 +261,7 @@ static NTSTATUS create_conn_struct_as_root(TALLOC_CTX *ctx,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	sconn->ev_ctx = samba_tevent_context_init(sconn);
-	if (sconn->ev_ctx == NULL) {
-		TALLOC_FREE(sconn);
-		return NT_STATUS_NO_MEMORY;
-	}
-
+	sconn->ev_ctx = ev;
 	sconn->msg_ctx = msg;
 
 	conn = conn_new(sconn);
@@ -402,6 +397,7 @@ NTSTATUS create_conn_struct_tos(struct messaging_context *msg,
 				struct conn_struct_tos **_c)
 {
 	struct conn_struct_tos *c = NULL;
+	struct tevent_context *ev = NULL;
 	NTSTATUS status;
 
 	*_c = NULL;
@@ -411,8 +407,15 @@ NTSTATUS create_conn_struct_tos(struct messaging_context *msg,
 		return NT_STATUS_NO_MEMORY;
 	}
 
+	ev = samba_tevent_context_init(c);
+	if (ev == NULL) {
+		TALLOC_FREE(c);
+		return NT_STATUS_NO_MEMORY;
+	}
+
 	become_root();
 	status = create_conn_struct_as_root(c,
+					    ev,
 					    msg,
 					    &c->conn,
 					    snum,
@@ -492,6 +495,44 @@ NTSTATUS create_conn_struct_tos_cwd(struct messaging_context *msg,
 	return NT_STATUS_OK;
 }
 
+/********************************************************
+ Fake up a connection struct for the VFS layer.
+ This takes an TALLOC_CTX and tevent_context from the
+ caller and the resulting connection_struct is stable
+ across the lifetime of mem_ctx and ev.
+
+ Note: this performs a vfs connect and changes cwd.
+
+ See also the comment for create_conn_struct_tos() above!
+*********************************************************/
+
+NTSTATUS create_conn_struct_cwd(TALLOC_CTX *mem_ctx,
+				struct tevent_context *ev,
+				struct messaging_context *msg,
+				const struct auth_session_info *session_info,
+				int snum,
+				const char *path,
+				struct connection_struct **c)
+{
+	NTSTATUS status;
+
+	become_root();
+	status = create_conn_struct_as_root(mem_ctx,
+					    ev,
+					    msg,
+					    c,
+					    snum,
+					    path,
+					    session_info);
+	unbecome_root();
+	if (!NT_STATUS_IS_OK(status)) {
+		TALLOC_FREE(c);
+		return status;
+	}
+
+	return NT_STATUS_OK;
+}
+
 static void shuffle_strlist(char **list, int count)
 {
 	int i;
@@ -705,9 +746,7 @@ static NTSTATUS dfs_path_lookup(TALLOC_CTX *ctx,
 
 	if (NT_STATUS_IS_OK(status)) {
 		/* XX_ALLOW_WCARD_XXX is called from search functions. */
-		if (ucf_flags &
-				(UCF_COND_ALLOW_WCARD_LCOMP|
-				 UCF_ALWAYS_ALLOW_WCARD_LCOMP)) {
+		if (ucf_flags & UCF_ALWAYS_ALLOW_WCARD_LCOMP) {
 			DEBUG(6,("dfs_path_lookup (FindFirst) No redirection "
 				 "for dfs link %s.\n", dfspath));
 			status = NT_STATUS_OK;
@@ -820,19 +859,17 @@ static NTSTATUS dfs_path_lookup(TALLOC_CTX *ctx,
  returned to the client.
 *****************************************************************/
 
-static NTSTATUS dfs_redirect(TALLOC_CTX *ctx,
+NTSTATUS dfs_redirect(TALLOC_CTX *ctx,
 			connection_struct *conn,
 			const char *path_in,
 			uint32_t ucf_flags,
 			bool allow_broken_path,
-			char **pp_path_out,
-			bool *ppath_contains_wcard)
+			char **pp_path_out)
 {
 	const struct loadparm_substitution *lp_sub =
 		loadparm_s3_global_substitution();
 	NTSTATUS status;
-	bool search_wcard_flag = (ucf_flags &
-		(UCF_COND_ALLOW_WCARD_LCOMP|UCF_ALWAYS_ALLOW_WCARD_LCOMP));
+	bool search_wcard_flag = (ucf_flags & UCF_ALWAYS_ALLOW_WCARD_LCOMP);
 	struct dfs_path *pdp = talloc(ctx, struct dfs_path);
 
 	if (!pdp) {
@@ -840,8 +877,7 @@ static NTSTATUS dfs_redirect(TALLOC_CTX *ctx,
 	}
 
 	status = parse_dfs_path(conn, path_in, search_wcard_flag,
-				allow_broken_path, pdp,
-			ppath_contains_wcard);
+				allow_broken_path, pdp);
 	if (!NT_STATUS_IS_OK(status)) {
 		TALLOC_FREE(pdp);
 		return status;
@@ -980,7 +1016,6 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 	struct connection_struct *conn = NULL;
 	int snum;
 	NTSTATUS status = NT_STATUS_NOT_FOUND;
-	bool dummy;
 	struct dfs_path *pdp = talloc_zero(frame, struct dfs_path);
 
 	if (!pdp) {
@@ -990,8 +1025,7 @@ NTSTATUS get_referred_path(TALLOC_CTX *ctx,
 
 	*self_referralp = False;
 
-	status = parse_dfs_path(NULL, dfs_path, False, allow_broken_path,
-				pdp, &dummy);
+	status = parse_dfs_path(NULL, dfs_path, False, allow_broken_path, pdp);
 	if (!NT_STATUS_IS_OK(status)) {
 		TALLOC_FREE(frame);
 		return status;
@@ -1233,15 +1267,13 @@ bool create_junction(TALLOC_CTX *ctx,
 	const struct loadparm_substitution *lp_sub =
 		loadparm_s3_global_substitution();
 	int snum;
-	bool dummy;
 	struct dfs_path *pdp = talloc(ctx,struct dfs_path);
 	NTSTATUS status;
 
 	if (!pdp) {
 		return False;
 	}
-	status = parse_dfs_path(NULL, dfs_path, False, allow_broken_path,
-				pdp, &dummy);
+	status = parse_dfs_path(NULL, dfs_path, False, allow_broken_path, pdp);
 	if (!NT_STATUS_IS_OK(status)) {
 		return False;
 	}
@@ -1808,36 +1840,3 @@ struct junction_map *enum_msdfs_links(TALLOC_CTX *ctx,
 	}
 	return jn;
 }
-
-/******************************************************************************
- Core function to resolve a dfs pathname possibly containing a wildcard.  If
- ppath_contains_wcard != NULL, it will be set to true if a wildcard is
- detected during dfs resolution.
-******************************************************************************/
-
-NTSTATUS resolve_dfspath_wcard(TALLOC_CTX *ctx,
-				connection_struct *conn,
-				const char *name_in,
-				uint32_t ucf_flags,
-				bool allow_broken_path,
-				char **pp_name_out,
-				bool *ppath_contains_wcard)
-{
-	bool path_contains_wcard = false;
-	NTSTATUS status = NT_STATUS_OK;
-
-	status = dfs_redirect(ctx,
-				conn,
-				name_in,
-				ucf_flags,
-				allow_broken_path,
-				pp_name_out,
-				&path_contains_wcard);
-
-	if (NT_STATUS_IS_OK(status) &&
-				ppath_contains_wcard != NULL &&
-				path_contains_wcard) {
-		*ppath_contains_wcard = path_contains_wcard;
-	}
-	return status;
-}
