@@ -17,7 +17,9 @@
 */
 
 #include <Python.h>
+#include "python/py3compat.h"
 #include "includes.h"
+#include "python/modules.h"
 #include "param/pyparam.h"
 #include "auth/gensec/gensec.h"
 #include "auth/gensec/gensec_internal.h" /* TODO: remove this */
@@ -43,7 +45,7 @@ static PyObject *py_get_name_by_authtype(PyObject *self, PyObject *args)
 	if (name == NULL)
 		Py_RETURN_NONE;
 
-	return PyString_FromString(name);
+	return PyUnicode_FromString(name);
 }
 
 static struct gensec_settings *settings_from_object(TALLOC_CTX *mem_ctx, PyObject *object)
@@ -71,7 +73,7 @@ static struct gensec_settings *settings_from_object(TALLOC_CTX *mem_ctx, PyObjec
 		return NULL;
 	}
 
-	s->target_hostname = PyString_AsString(py_hostname);
+	s->target_hostname = PyUnicode_AsUTF8(py_hostname);
 	s->lp_ctx = lpcfg_from_py_object(s, py_lp_ctx);
 	return s;
 }
@@ -175,11 +177,19 @@ static PyObject *py_gensec_start_server(PyTypeObject *type, PyObject *args, PyOb
 	}
 
 	if (py_auth_context != Py_None) {
-		auth_context = pytalloc_get_type(py_auth_context, struct auth4_context);
+		bool ok = py_check_dcerpc_type(py_auth_context,
+					       "samba.auth",
+					       "AuthContext");
+		if (!ok) {
+			return NULL;
+		}
+
+		auth_context = pytalloc_get_type(py_auth_context,
+						 struct auth4_context);
 		if (!auth_context) {
 			PyErr_Format(PyExc_TypeError,
 				     "Expected auth.AuthContext for auth_context argument, got %s",
-				     talloc_get_name(pytalloc_get_ptr(py_auth_context)));
+				     pytalloc_get_name(py_auth_context));
 			return NULL;
 		}
 	}
@@ -240,6 +250,25 @@ static PyObject *py_gensec_set_target_service(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
+static PyObject *py_gensec_set_target_service_description(PyObject *self, PyObject *args)
+{
+	struct gensec_security *security = pytalloc_get_type(self, struct gensec_security);
+	char *target_service_description;
+	NTSTATUS status;
+
+	if (!PyArg_ParseTuple(args, "s", &target_service_description))
+		return NULL;
+
+	status = gensec_set_target_service_description(security,
+						       target_service_description);
+	if (!NT_STATUS_IS_OK(status)) {
+		PyErr_SetNTSTATUS(status);
+		return NULL;
+	}
+
+	Py_RETURN_NONE;
+}
+
 static PyObject *py_gensec_set_credentials(PyObject *self, PyObject *args)
 {
 	PyObject *py_creds = Py_None;
@@ -252,9 +281,11 @@ static PyObject *py_gensec_set_credentials(PyObject *self, PyObject *args)
 
 	creds = PyCredentials_AsCliCredentials(py_creds);
 	if (!creds) {
-		PyErr_Format(PyExc_TypeError,
-			     "Expected samba.credentaials for credentials argument got  %s",
-			     talloc_get_name(pytalloc_get_ptr(py_creds)));
+		PyErr_Format(
+			PyExc_TypeError,
+			"Expected samba.credentials for credentials argument, "
+			"got %s", pytalloc_get_name(py_creds));
+		return NULL;
 	}
 
 	status = gensec_set_credentials(security, creds);
@@ -266,7 +297,8 @@ static PyObject *py_gensec_set_credentials(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
-static PyObject *py_gensec_session_info(PyObject *self)
+static PyObject *py_gensec_session_info(PyObject *self,
+		PyObject *Py_UNUSED(ignored))
 {
 	TALLOC_CTX *mem_ctx;
 	NTSTATUS status;
@@ -291,7 +323,8 @@ static PyObject *py_gensec_session_info(PyObject *self)
 	return py_session_info;
 }
 
-static PyObject *py_gensec_session_key(PyObject *self)
+static PyObject *py_gensec_session_key(PyObject *self,
+		PyObject *Py_UNUSED(ignored))
 {
 	TALLOC_CTX *mem_ctx;
 	NTSTATUS status;
@@ -312,7 +345,7 @@ static PyObject *py_gensec_session_key(PyObject *self)
 		return NULL;
 	}
 
-	session_key_obj = PyString_FromStringAndSize((const char *)session_key.data,
+	session_key_obj = PyBytes_FromStringAndSize((const char *)session_key.data,
 						     session_key.length);
 	talloc_free(mem_ctx);
 	return session_key_obj;
@@ -411,12 +444,13 @@ static PyObject *py_gensec_set_max_update_size(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
-static PyObject *py_gensec_max_update_size(PyObject *self)
+static PyObject *py_gensec_max_update_size(PyObject *self,
+		PyObject *Py_UNUSED(ignored))
 {
 	struct gensec_security *security = pytalloc_get_type(self, struct gensec_security);
 	unsigned int max_update_size = gensec_max_update_size(security);
 
-	return PyInt_FromLong(max_update_size);
+	return PyLong_FromLong(max_update_size);
 }
 
 static PyObject *py_gensec_update(PyObject *self, PyObject *args)
@@ -424,7 +458,7 @@ static PyObject *py_gensec_update(PyObject *self, PyObject *args)
 	NTSTATUS status;
 	TALLOC_CTX *mem_ctx;
 	DATA_BLOB in, out;
-	PyObject *ret, *py_in;
+	PyObject *py_bytes, *result, *py_in;
 	struct gensec_security *security = pytalloc_get_type(self, struct gensec_security);
 	PyObject *finished_processing;
 
@@ -432,14 +466,13 @@ static PyObject *py_gensec_update(PyObject *self, PyObject *args)
 		return NULL;
 
 	mem_ctx = talloc_new(NULL);
-
-	if (!PyString_Check(py_in)) {
-		PyErr_Format(PyExc_TypeError, "expected a string");
+	if (!PyBytes_Check(py_in)) {
+		PyErr_Format(PyExc_TypeError, "bytes expected");
 		return NULL;
 	}
 
-	in.data = (uint8_t *)PyString_AsString(py_in);
-	in.length = PyString_Size(py_in);
+	in.data = (uint8_t *)PyBytes_AsString(py_in);
+	in.length = PyBytes_Size(py_in);
 
 	status = gensec_update(security, mem_ctx, in, &out);
 
@@ -449,7 +482,8 @@ static PyObject *py_gensec_update(PyObject *self, PyObject *args)
 		talloc_free(mem_ctx);
 		return NULL;
 	}
-	ret = PyString_FromStringAndSize((const char *)out.data, out.length);
+	py_bytes = PyBytes_FromStringAndSize((const char *)out.data,
+					     out.length);
 	talloc_free(mem_ctx);
 
 	if (NT_STATUS_EQUAL(status, NT_STATUS_MORE_PROCESSING_REQUIRED)) {
@@ -458,7 +492,9 @@ static PyObject *py_gensec_update(PyObject *self, PyObject *args)
 		finished_processing = Py_True;
 	}
 
-	return PyTuple_Pack(2, finished_processing, ret);
+	result = PyTuple_Pack(2, finished_processing, py_bytes);
+	Py_XDECREF(py_bytes);
+	return result;
 }
 
 static PyObject *py_gensec_wrap(PyObject *self, PyObject *args)
@@ -475,12 +511,12 @@ static PyObject *py_gensec_wrap(PyObject *self, PyObject *args)
 
 	mem_ctx = talloc_new(NULL);
 
-	if (!PyString_Check(py_in)) {
-		PyErr_Format(PyExc_TypeError, "expected a string");
+	if (!PyBytes_Check(py_in)) {
+		PyErr_Format(PyExc_TypeError, "bytes expected");
 		return NULL;
 	}
-	in.data = (uint8_t *)PyString_AsString(py_in);
-	in.length = PyString_Size(py_in);
+	in.data = (uint8_t *)PyBytes_AsString(py_in);
+	in.length = PyBytes_Size(py_in);
 
 	status = gensec_wrap(security, mem_ctx, &in, &out);
 
@@ -490,11 +526,12 @@ static PyObject *py_gensec_wrap(PyObject *self, PyObject *args)
 		return NULL;
 	}
 
-	ret = PyString_FromStringAndSize((const char *)out.data, out.length);
+	ret = PyBytes_FromStringAndSize((const char *)out.data, out.length);
 	talloc_free(mem_ctx);
 	return ret;
 }
 
+
 static PyObject *py_gensec_unwrap(PyObject *self, PyObject *args)
 {
 	NTSTATUS status;
@@ -509,13 +546,13 @@ static PyObject *py_gensec_unwrap(PyObject *self, PyObject *args)
 
 	mem_ctx = talloc_new(NULL);
 
-	if (!PyString_Check(py_in)) {
-		PyErr_Format(PyExc_TypeError, "expected a string");
+	if (!PyBytes_Check(py_in)) {
+		PyErr_Format(PyExc_TypeError, "bytes expected");
 		return NULL;
 	}
 
-	in.data = (uint8_t *)PyString_AsString(py_in);
-	in.length = PyString_Size(py_in);
+	in.data = (uint8_t *)PyBytes_AsString(py_in);
+	in.length = PyBytes_Size(py_in);
 
 	status = gensec_unwrap(security, mem_ctx, &in, &out);
 
@@ -525,7 +562,7 @@ static PyObject *py_gensec_unwrap(PyObject *self, PyObject *args)
 		return NULL;
 	}
 
-	ret = PyString_FromStringAndSize((const char *)out.data, out.length);
+	ret = PyBytes_FromStringAndSize((const char *)out.data, out.length);
 	talloc_free(mem_ctx);
 	return ret;
 }
@@ -608,16 +645,22 @@ static PyObject *py_gensec_check_packet(PyObject *self, PyObject *args)
 }
 
 static PyMethodDef py_gensec_security_methods[] = {
-	{ "start_client", (PyCFunction)py_gensec_start_client, METH_VARARGS|METH_KEYWORDS|METH_CLASS, 
+	{ "start_client", PY_DISCARD_FUNC_SIG(PyCFunction,
+					      py_gensec_start_client),
+		METH_VARARGS|METH_KEYWORDS|METH_CLASS,
 		"S.start_client(settings) -> gensec" },
-	{ "start_server", (PyCFunction)py_gensec_start_server, METH_VARARGS|METH_KEYWORDS|METH_CLASS,
+	{ "start_server", PY_DISCARD_FUNC_SIG(PyCFunction,
+					      py_gensec_start_server),
+		METH_VARARGS|METH_KEYWORDS|METH_CLASS,
 		"S.start_server(auth_ctx, settings) -> gensec" },
 	{ "set_credentials", (PyCFunction)py_gensec_set_credentials, METH_VARARGS, 
 		"S.start_client(credentials)" },
 	{ "set_target_hostname", (PyCFunction)py_gensec_set_target_hostname, METH_VARARGS, 
-		"S.start_target_hostname(target_hostname)" },
+		"S.start_target_hostname(target_hostname) \n This sets the Kerberos target hostname to obtain a ticket for." },
 	{ "set_target_service", (PyCFunction)py_gensec_set_target_service, METH_VARARGS, 
-		"S.start_target_service(target_service)" },
+		"S.start_target_service(target_service) \n This sets the Kerberos target service to obtain a ticket for.  The default value is 'host'" },
+	{ "set_target_service_description", (PyCFunction)py_gensec_set_target_service_description, METH_VARARGS,
+		"S.start_target_service_description(target_service_description) \n This description is set server-side and used in authentication and authorization logs.  The default value is that provided to set_target_service() or None."},
 	{ "session_info", (PyCFunction)py_gensec_session_info, METH_NOARGS,
 		"S.session_info() -> info" },
 	{ "session_key", (PyCFunction)py_gensec_session_key, METH_NOARGS,
@@ -636,7 +679,7 @@ static PyMethodDef py_gensec_security_methods[] = {
 		"S.have_feature()\n Return True if GENSEC negotiated a particular feature." },
 	{ "set_max_update_size",  (PyCFunction)py_gensec_set_max_update_size, METH_VARARGS,
 		"S.set_max_update_size(max_size) \n Some mechs can fragment update packets, needs to be use before the mech is started." },
-	{ "max_update_size",  (PyCFunction)py_gensec_max_update_size, 0,
+	{ "max_update_size",  (PyCFunction)py_gensec_max_update_size, METH_NOARGS,
 		"S.max_update_size() \n Return the current max_update_size." },
 	{ "update",  (PyCFunction)py_gensec_update, METH_VARARGS,
 		"S.update(blob_in) -> (finished, blob_out)\nPerform one step in a GENSEC dance.  Repeat with new packets until finished is true or exception." },
@@ -650,7 +693,14 @@ static PyMethodDef py_gensec_security_methods[] = {
 		"S.sign_packet(data, whole_pdu) -> sig\nSign a DCERPC packet." },
 	{ "check_packet",  (PyCFunction)py_gensec_check_packet, METH_VARARGS,
 		"S.check_packet(data, whole_pdu, sig)\nCheck a DCERPC packet." },
-	{ NULL }
+	{0}
+};
+
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    .m_name = "gensec",
+    .m_doc = "Generic Security Interface.",
+    .m_size = -1,
 };
 
 static PyTypeObject Py_Security = {
@@ -659,27 +709,28 @@ static PyTypeObject Py_Security = {
 	.tp_methods = py_gensec_security_methods,
 };
 
-void initgensec(void);
-void initgensec(void)
+MODULE_INIT_FUNC(gensec)
 {
 	PyObject *m;
 
 	if (pytalloc_BaseObject_PyType_Ready(&Py_Security) < 0)
-		return;
+		return NULL;
 
-	m = Py_InitModule3("gensec", NULL, "Generic Security Interface.");
+	m = PyModule_Create(&moduledef);
 	if (m == NULL)
-		return;
+		return NULL;
 
-	PyModule_AddObject(m, "FEATURE_SESSION_KEY",     PyInt_FromLong(GENSEC_FEATURE_SESSION_KEY));
-	PyModule_AddObject(m, "FEATURE_SIGN",            PyInt_FromLong(GENSEC_FEATURE_SIGN));
-	PyModule_AddObject(m, "FEATURE_SEAL",            PyInt_FromLong(GENSEC_FEATURE_SEAL));
-	PyModule_AddObject(m, "FEATURE_DCE_STYLE",       PyInt_FromLong(GENSEC_FEATURE_DCE_STYLE));
-	PyModule_AddObject(m, "FEATURE_ASYNC_REPLIES",   PyInt_FromLong(GENSEC_FEATURE_ASYNC_REPLIES));
-	PyModule_AddObject(m, "FEATURE_DATAGRAM_MODE",   PyInt_FromLong(GENSEC_FEATURE_DATAGRAM_MODE));
-	PyModule_AddObject(m, "FEATURE_SIGN_PKT_HEADER", PyInt_FromLong(GENSEC_FEATURE_SIGN_PKT_HEADER));
-	PyModule_AddObject(m, "FEATURE_NEW_SPNEGO",      PyInt_FromLong(GENSEC_FEATURE_NEW_SPNEGO));
+	PyModule_AddObject(m, "FEATURE_SESSION_KEY",     PyLong_FromLong(GENSEC_FEATURE_SESSION_KEY));
+	PyModule_AddObject(m, "FEATURE_SIGN",            PyLong_FromLong(GENSEC_FEATURE_SIGN));
+	PyModule_AddObject(m, "FEATURE_SEAL",            PyLong_FromLong(GENSEC_FEATURE_SEAL));
+	PyModule_AddObject(m, "FEATURE_DCE_STYLE",       PyLong_FromLong(GENSEC_FEATURE_DCE_STYLE));
+	PyModule_AddObject(m, "FEATURE_ASYNC_REPLIES",   PyLong_FromLong(GENSEC_FEATURE_ASYNC_REPLIES));
+	PyModule_AddObject(m, "FEATURE_DATAGRAM_MODE",   PyLong_FromLong(GENSEC_FEATURE_DATAGRAM_MODE));
+	PyModule_AddObject(m, "FEATURE_SIGN_PKT_HEADER", PyLong_FromLong(GENSEC_FEATURE_SIGN_PKT_HEADER));
+	PyModule_AddObject(m, "FEATURE_NEW_SPNEGO",      PyLong_FromLong(GENSEC_FEATURE_NEW_SPNEGO));
 
 	Py_INCREF(&Py_Security);
 	PyModule_AddObject(m, "Security", (PyObject *)&Py_Security);
+
+	return m;
 }
