@@ -1,6 +1,7 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # Originally based on ./sam.py
+from __future__ import print_function
 from unicodedata import normalize
 import locale
 locale.setlocale(locale.LC_ALL, ('en_US', 'UTF-8'))
@@ -13,7 +14,8 @@ import re
 sys.path.insert(0, "bin/python")
 import samba
 from samba.tests.subunitrun import SubunitOptions, TestProgram
-
+from samba.common import cmp
+from functools import cmp_to_key
 import samba.getopt as options
 
 from samba.auth import system_session
@@ -39,15 +41,25 @@ if len(args) < 1:
     parser.print_usage()
     sys.exit(1)
 
-host = args[0]
+datadir = os.getenv("DATA_DIR", None)
+if not datadir:
+    print("Please specify the location of the sort expected results with env variable DATA_DIR")
+    sys.exit(1)
+
+host = os.getenv("SERVER", None)
+if not host:
+    print("Please specify the host with env variable SERVER")
+    sys.exit(1)
 
 lp = sambaopts.get_loadparm()
 creds = credopts.get_credentials(lp)
 
 
 def norm(x):
-    x = x.decode('utf-8')
-    return normalize('NFKC', x).upper().encode('utf-8')
+    if not isinstance(x, str):
+        x = x.decode('utf8')
+    return normalize('NFKC', x).upper()
+
 
 # Python, Windows, and Samba all sort the following sequence in
 # drastically different ways. The order here is what you get from
@@ -76,7 +88,9 @@ class BaseSortTests(samba.tests.TestCase):
             "objectclass": "user",
             'givenName': "abcdefghijklmnopqrstuvwxyz"[i % 26],
             "roomNumber": "%sb\x00c" % (n - i),
-            "carLicense": "后来经",
+            # with python3 re.sub(r'[^\w,.]', repl, string) doesn't
+            # work as expected with unicode as value for carLicense
+            "carLicense": "XXXXXXXXX" if self.avoid_tricky_sort else "后来经",
             "employeeNumber": "%s%sx" % (abs(i * (99 - i)), '\n' * (i & 255)),
             "accountExpires": "%s" % (10 ** 9 + 1000000 * i),
             "msTSExpireDate4": "19%02d0101010000.0Z" % (i % 100),
@@ -102,7 +116,7 @@ class BaseSortTests(samba.tests.TestCase):
                                                                  chr(i & 255),
                                                                  i),
                 "displayNamePrintable": "%d\x00%c" % (i, i & 255),
-                "adminDisplayName": "%d\x00b" % (n-i),
+                "adminDisplayName": "%d\x00b" % (n - i),
                 "title": "%d%sb" % (n - i, '\x00' * i),
 
                 # Names that vary only in case. Windows returns
@@ -133,8 +147,8 @@ class BaseSortTests(samba.tests.TestCase):
         if False:
             try:
                 self.ldb.delete(self.ou, ['tree_delete:1'])
-            except ldb.LdbError, e:
-                print "tried deleting %s, got error %s" % (self.ou, e)
+            except ldb.LdbError as e:
+                print("tried deleting %s, got error %s" % (self.ou, e))
 
         self.ldb.add({
             "dn": self.ou,
@@ -166,44 +180,31 @@ class BaseSortTests(samba.tests.TestCase):
         self.expected_results = {}
         self.expected_results_binary = {}
 
-        for k in self.locale_sorted_keys:
-            # Using key=locale.strxfrm fails on \x00
-            forward = sorted((norm(x[k]) for x in self.users),
-                             cmp=locale.strcoll)
-            reverse = list(reversed(forward))
-            self.expected_results[k] = (forward, reverse)
-
         for k in self.binary_sorted_keys:
             forward = sorted((x[k] for x in self.users))
             reverse = list(reversed(forward))
             self.expected_results_binary[k] = (forward, reverse)
-            self.expected_results[k] = (forward, reverse)
 
-        # Fix up some because Python gets it wrong, using Schwartzian tramsform
-        for k in ('adminDisplayName', 'title', 'streetAddress',
-                  'employeeNumber'):
-            if k in self.expected_results:
-                broken = self.expected_results[k][0]
-                tmp = [(x.replace('\x00', ''), x) for x in broken]
-                tmp.sort()
-                fixed = [x[1] for x in tmp]
-                self.expected_results[k] = (fixed, list(reversed(fixed)))
-        for k in ('streetAddress', 'postalAddress'):
-            if k in self.expected_results:
-                c = {}
-                for u in self.users:
-                    x = u[k]
-                    if x in c:
-                        c[x] += 1
-                        continue
-                    c[x] = 1
-                fixed = []
-                for x in FIENDISH_TESTS:
-                    fixed += [norm(x)] * c.get(x, 0)
-
-                rev = list(reversed(fixed))
-                self.expected_results[k] = (fixed, rev)
+        # FYI: Expected result data was generated from the old
+        # code that was manually sorting (while executing with
+        # python2)
+        # The resulting data was injected into the data file with
+        # code similar to:
+        #
+        # for k in self.expected_results:
+        #     f.write("%s = %s\n" % (k,  repr(self.expected_results[k][0])))
 
+        f = open(self.results_file, "r")
+        for line in f:
+            if len(line.split('=', 1)) == 2:
+                key = line.split('=', 1)[0].strip()
+                value = line.split('=', 1)[1].strip()
+                if value.startswith('['):
+                    import ast
+                    fwd_list = ast.literal_eval(value)
+                    rev_list = list(reversed(fwd_list))
+                    self.expected_results[key] = (fwd_list, rev_list)
+        f.close()
     def tearDown(self):
         super(BaseSortTests, self).tearDown()
         self.ldb.delete(self.ou, ['tree_delete:1'])
@@ -222,13 +223,13 @@ class BaseSortTests(samba.tests.TestCase):
                 expected_order = self.expected_results[attr][rev]
                 received_order = [norm(x[attr][0]) for x in res]
                 if expected_order != received_order:
-                    print attr, ['forward', 'reverse'][rev]
-                    print "expected", expected_order
-                    print "recieved", received_order
-                    print "unnormalised:", [x[attr][0] for x in res]
-                    print "unnormalised: «%s»" % '»  «'.join(x[attr][0]
-                                                             for x in res)
-                self.assertEquals(expected_order, received_order)
+                    print(attr, ['forward', 'reverse'][rev])
+                    print("expected", expected_order)
+                    print("received", received_order)
+                    print("unnormalised:", [x[attr][0] for x in res])
+                    print("unnormalised: «%s»" % '»  «'.join(str(x[attr][0])
+                                                             for x in res))
+                self.assertEqual(expected_order, received_order)
 
     def _test_server_sort_binary(self):
         for attr in self.binary_sorted_keys:
@@ -240,12 +241,12 @@ class BaseSortTests(samba.tests.TestCase):
 
                 self.assertEqual(len(res), len(self.users))
                 expected_order = self.expected_results_binary[attr][rev]
-                received_order = [x[attr][0] for x in res]
+                received_order = [str(x[attr][0]) for x in res]
                 if expected_order != received_order:
-                    print attr
-                    print expected_order
-                    print received_order
-                self.assertEquals(expected_order, received_order)
+                    print(attr)
+                    print(expected_order)
+                    print(received_order)
+                self.assertEqual(expected_order, received_order)
 
     def _test_server_sort_us_english(self):
         # Windows doesn't support many matching rules, but does allow
@@ -268,15 +269,15 @@ class BaseSortTests(samba.tests.TestCase):
                     expected_order = self.expected_results[attr][rev]
                     received_order = [norm(x[attr][0]) for x in res]
                     if expected_order != received_order:
-                        print attr, lang
-                        print ['forward', 'reverse'][rev]
-                        print "expected: ", expected_order
-                        print "recieved: ", received_order
-                        print "unnormalised:", [x[attr][0] for x in res]
-                        print "unnormalised: «%s»" % '»  «'.join(x[attr][0]
-                                                                 for x in res)
+                        print(attr, lang)
+                        print(['forward', 'reverse'][rev])
+                        print("expected: ", expected_order)
+                        print("received: ", received_order)
+                        print("unnormalised:", [x[attr][0] for x in res])
+                        print("unnormalised: «%s»" % '»  «'.join(str(x[attr][0])
+                                                                 for x in res))
 
-                    self.assertEquals(expected_order, received_order)
+                    self.assertEqual(expected_order, received_order)
 
     def _test_server_sort_different_attr(self):
 
@@ -296,13 +297,13 @@ class BaseSortTests(samba.tests.TestCase):
                           "employeeNumber": cmp_locale,
                           "accountExpires": cmp_numeric,
                           "msTSExpireDate4": cmp_binary}
-        attrs = sort_functions.keys()
+        attrs = list(sort_functions.keys())
         attr_pairs = zip(attrs, attrs[1:] + attrs[:1])
 
         for sort_attr, result_attr in attr_pairs:
             forward = sorted(((norm(x[sort_attr]), norm(x[result_attr]))
                              for x in self.users),
-                             cmp=sort_functions[sort_attr])
+                             key=cmp_to_key(sort_functions[sort_attr]))
             reverse = list(reversed(forward))
 
             for rev in (0, 1):
@@ -318,24 +319,24 @@ class BaseSortTests(samba.tests.TestCase):
                 received_order = [norm(x[result_attr][0]) for x in res]
 
                 if expected_order != received_order:
-                    print sort_attr, result_attr, ['forward', 'reverse'][rev]
-                    print "expected", expected_order
-                    print "recieved", received_order
-                    print "unnormalised:", [x[result_attr][0] for x in res]
-                    print "unnormalised: «%s»" % '»  «'.join(x[result_attr][0]
-                                                             for x in res)
-                    print "pairs:", pairs
+                    print(sort_attr, result_attr, ['forward', 'reverse'][rev])
+                    print("expected", expected_order)
+                    print("received", received_order)
+                    print("unnormalised:", [x[result_attr][0] for x in res])
+                    print("unnormalised: «%s»" % '»  «'.join(str(x[result_attr][0])
+                                                             for x in res))
+                    print("pairs:", pairs)
                     # There are bugs in Windows that we don't want (or
                     # know how) to replicate regarding timestamp sorting.
                     # Let's remind ourselves.
                     if result_attr == "msTSExpireDate4":
-                        print '-' * 72
-                        print ("This test fails against Windows with the "
-                               "default number of elements (33).")
-                        print "Try with --elements=27 (or similar)."
-                        print '-' * 72
+                        print('-' * 72)
+                        print("This test fails against Windows with the "
+                              "default number of elements (33).")
+                        print("Try with --elements=27 (or similar).")
+                        print('-' * 72)
 
-                self.assertEquals(expected_order, received_order)
+                self.assertEqual(expected_order, received_order)
                 for x in res:
                     if sort_attr in x:
                         self.fail('the search for %s should not return %s' %
@@ -344,6 +345,7 @@ class BaseSortTests(samba.tests.TestCase):
 
 class SimpleSortTests(BaseSortTests):
     avoid_tricky_sort = True
+    results_file = os.path.join(datadir, "simplesort.expected")
     def test_server_sort_different_attr(self):
         self._test_server_sort_different_attr()
 
@@ -359,6 +361,7 @@ class SimpleSortTests(BaseSortTests):
 
 class UnicodeSortTests(BaseSortTests):
     avoid_tricky_sort = False
+    results_file = os.path.join(datadir, "unicodesort.expected")
 
     def test_server_sort_default(self):
         self._test_server_sort_default()
@@ -375,6 +378,3 @@ if "://" not in host:
         host = "tdb://%s" % host
     else:
         host = "ldap://%s" % host
-
-
-TestProgram(module=__name__, opts=subunitopts)
