@@ -152,7 +152,7 @@ static PyMethodDef py_xattr_methods[] = {
 		"Set the given attribute to the given value on the given file." },
 	{ "is_xattr_supported", (PyCFunction)py_is_xattr_supported, METH_NOARGS,
 		"Return true if xattr are supported on this system\n"},
-	{ NULL }
+	{0}
 };
 
 static struct PyModuleDef moduledef = {
