@@ -27,6 +27,7 @@
 #include "lib/param/loadparm.h"
 #include "lib/param/param_global.h"
 #include "dynconfig.h"
+#include "lib/util/time.h"
 
 /* test string to compare with when debug_callback is called */
 #define TEST_STRING "smbc_setLogCallback test"
@@ -53,17 +54,27 @@ static void auth_callback(const char *srv,
 		cli_credentials_get_username(popt_get_cmdline_credentials());
 	const char *password =
 		cli_credentials_get_password(popt_get_cmdline_credentials());
+	ssize_t ret;
 
 	if (workgroup != NULL) {
-		snprintf(wg, wglen, "%s", workgroup);
+		ret = strlcpy(wg, workgroup, wglen);
+		if (ret >= wglen) {
+			abort();
+		}
 	}
 
 	if (username != NULL) {
-		snprintf(un, unlen, "%s", username);
+		ret = strlcpy(un, username, unlen);
+		if (ret >= unlen) {
+			abort();
+		}
 	}
 
 	if (password != NULL) {
-		snprintf(pw, pwlen, "%s", password);
+		ret = strlcpy(pw, password, pwlen);
+		if (ret >= pwlen) {
+			abort();
+		}
 	}
 };
 
@@ -79,6 +90,7 @@ bool torture_libsmbclient_init_context(struct torture_context *tctx,
 	SMBCCTX *ctx = NULL;
 	SMBCCTX *p = NULL;
 	bool ok = true;
+	int dbglevel = DEBUGLEVEL;
 
 	ctx = smbc_new_context();
 	torture_assert_not_null_goto(tctx,
@@ -94,7 +106,7 @@ bool torture_libsmbclient_init_context(struct torture_context *tctx,
 				     out,
 				     "Failed to initialize context");
 
-	smbc_setDebug(ctx, DEBUGLEVEL);
+	smbc_setDebug(ctx, dbglevel);
 	smbc_setOptionDebugToStderr(ctx, 1);
 
 	if (workgroup != NULL) {
@@ -165,9 +177,10 @@ static bool torture_libsmbclient_setConfiguration(struct torture_context *tctx)
 				"");
 
 	ctx = smbc_new_context();
-	torture_assert(tctx, ctx, "failed to get new context");
+	torture_assert_not_null(tctx, ctx, "failed to get new context");
 
-	torture_assert(tctx, smbc_init_context(ctx), "failed to init context");
+	torture_assert_not_null(
+		tctx, smbc_init_context(ctx), "failed to init context");
 
 	torture_comment(tctx, "Testing smbc_setConfiguration - new file %s\n",
 		new_smb_conf);
@@ -243,7 +256,7 @@ static bool test_opendir(struct torture_context *tctx,
 
 static bool torture_libsmbclient_opendir(struct torture_context *tctx)
 {
-	int i;
+	size_t i;
 	SMBCCTX *ctx;
 	bool ret = true;
 	const char *bad_urls[] = {
@@ -400,11 +413,11 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 	struct stat st = {0};
 	struct stat st2 = {0};
 
-	if (smburl == NULL) {
-		torture_fail(tctx,
-			"option --option=torture:smburl="
-			"smb://user:password@server/share missing\n");
-	}
+	torture_assert_not_null(
+		tctx,
+		smburl,
+		"option --option=torture:smburl="
+		"smb://user:password@server/share missing\n");
 
 	DEBUG(0,("torture_libsmbclient_readdirplus_seek start\n"));
 
@@ -414,31 +427,22 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 	dname = talloc_asprintf(tctx,
 				"%s/rd_seek",
 				smburl);
-	if (dname == NULL) {
-		torture_fail_goto(tctx,
-			done,
-			"talloc fail\n");
-	}
+	torture_assert_not_null_goto(
+		tctx, dname, success, done, "talloc fail\n");
 
 	/* Ensure the files don't exist. */
 	for (i = 0; i < 100; i++) {
 		filename[i] = talloc_asprintf(tctx,
 				"test_readdirplus_%u.txt",
 				i);
-		if (filename[i] == NULL) {
-			torture_fail_goto(tctx,
-				done,
-				"talloc fail\n");
-		}
+		torture_assert_not_null_goto(
+			tctx, filename[i], success, done, "talloc fail");
 		full_filename[i] = talloc_asprintf(tctx,
 				"%s/%s",
 				dname,
 				filename[i]);
-		if (full_filename[i] == NULL) {
-			torture_fail_goto(tctx,
-				done,
-				"talloc fail\n");
-		}
+		torture_assert_not_null_goto(
+			tctx, full_filename[i], success, done, "talloc fail");
 		(void)smbc_unlink(full_filename[i]);
 	}
 	/* Ensure the directory doesn't exist. */
@@ -446,14 +450,15 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 
 	/* Create containing directory. */
 	ret = smbc_mkdir(dname, 0777);
-	if (ret != 0) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
+	torture_assert_goto(
+		tctx,
+		ret == 0,
+		success,
+		done,
+		talloc_asprintf(tctx,
 				"failed to create directory '%s': %s",
 				dname,
 				strerror(errno)));
-	}
 
 	DEBUG(0,("torture_libsmbclient_readdirplus_seek create\n"));
 
@@ -483,15 +488,16 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 
 	/* Now enumerate the directory. */
 	dhandle = smbc_opendir(dname);
-	if (dhandle < 0) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
+	torture_assert_goto(
+		tctx,
+		dhandle >= 0,
+		success,
+		done,
+		talloc_asprintf(tctx,
 				"failed to obtain "
 				"directory handle for '%s' : %s",
 				dname,
 				strerror(errno)));
-	}
 
 	/* Read all the files. 100 we created plus . and .. */
 	for (i = 0; i < 102; i++) {
@@ -506,13 +512,14 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 		/* Store at offset 50. */
 		if (i == 50) {
 			telldir_50 = smbc_telldir(dhandle);
-			if (telldir_50 == (off_t)-1) {
-				torture_fail_goto(tctx,
-					done,
-					talloc_asprintf(tctx,
+			torture_assert_goto(
+				tctx,
+				telldir_50 != (off_t)-1,
+				success,
+				done,
+				talloc_asprintf(tctx,
 						"telldir failed file %s\n",
 						direntries[i]->name));
-			}
 		}
 
 		if (ISDOT(direntries[i]->name)) {
@@ -529,13 +536,14 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 				found = true;
 			}
 		}
-		if (!found) {
-			torture_fail_goto(tctx,
-				done,
-				talloc_asprintf(tctx,
+		torture_assert_goto(
+			tctx,
+			found,
+			success,
+			done,
+			talloc_asprintf(tctx,
 					"failed to find file %s\n",
 					direntries[i]->name));
-		}
 	}
 
 	/*
@@ -559,15 +567,16 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 	for (i = 51; i < 102; i++) {
 		const struct libsmb_file_info *entry =
 				smbc_readdirplus(dhandle);
-		if (entry != direntries[i]) {
-			torture_fail_goto(tctx,
-				done,
-				talloc_asprintf(tctx,
+		torture_assert_goto(
+			tctx,
+			entry == direntries[i],
+			success,
+			done,
+			talloc_asprintf(tctx,
 					"after seek - failed to find "
 					"file %s - got %s\n",
 					direntries[i]->name,
 					entry->name));
-		}
 	}
 
 	/* Seek back to the start. */
@@ -596,6 +605,12 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 	getdentries = (struct smbc_dirent *)talloc_array_size(tctx,
 						getdentries_size,
 						1);
+	torture_assert_not_null_goto(
+		tctx,
+		getdentries,
+		success,
+		done,
+		"talloc fail");
 
 	ret = smbc_getdents(dhandle, getdentries, getdentries_size);
 	torture_assert_goto(tctx,
@@ -607,12 +622,13 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 			dname));
 
 	telldir_20 = smbc_telldir(dhandle);
-	if (telldir_20 == (off_t)-1) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
-				"telldir (20) failed\n"));
-	}
+	torture_assert_goto(
+		tctx,
+		telldir_20 != (off_t)-1,
+		success,
+		done,
+		"telldir (20) failed\n");
+
 	/* Read another 20. */
 	ret = smbc_getdents(dhandle, getdentries, getdentries_size);
 	torture_assert_goto(tctx,
@@ -636,24 +652,25 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 
 	/* Read with readdir. */
 	dirent_20 = smbc_readdir(dhandle);
-	if (dirent_20 == NULL) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
-				"smbc_readdir (20) failed\n"));
-	}
+	torture_assert_not_null_goto(
+		tctx,
+		dirent_20,
+		success,
+		done,
+		"smbc_readdir (20) failed\n");
 
 	/* Ensure the getdents and readdir names are the same. */
 	ret = strcmp(dirent_20->name, getdentries[0].name);
-	if (ret != 0) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
+	torture_assert_goto(
+		tctx,
+		ret == 0,
+		success,
+		done,
+		talloc_asprintf(tctx,
 				"after seek (20) readdir name missmatch "
 				"file %s - got %s\n",
 				dirent_20->name,
 				getdentries[0].name));
-	}
 
 	/* Seek back to 20. */
 	ret = smbc_lseekdir(dhandle, telldir_20);
@@ -667,24 +684,25 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 			dname));
 	/* Read with readdirplus. */
 	direntries_20 = smbc_readdirplus(dhandle);
-	if (direntries_20 == NULL) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
-				"smbc_readdirplus (20) failed\n"));
-	}
+	torture_assert_not_null_goto(
+		tctx,
+		direntries_20,
+		success,
+		done,
+		"smbc_readdirplus (20) failed\n");
 
 	/* Ensure the readdirplus and readdir names are the same. */
 	ret = strcmp(dirent_20->name, direntries_20->name);
-	if (ret != 0) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
+	torture_assert_goto(
+		tctx,
+		ret == 0,
+		success,
+		done,
+		talloc_asprintf(tctx,
 				"after seek (20) readdirplus name missmatch "
 				"file %s - got %s\n",
 				dirent_20->name,
 				direntries_20->name));
-	}
 
 	/* Seek back to 20. */
 	ret = smbc_lseekdir(dhandle, telldir_20);
@@ -699,35 +717,37 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 
 	/* Read with readdirplus2. */
 	direntriesplus_20 = smbc_readdirplus2(dhandle, &st2);
-	if (direntriesplus_20 == NULL) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
-				"smbc_readdirplus2 (20) failed\n"));
-	}
+	torture_assert_not_null_goto(
+		tctx,
+		direntriesplus_20,
+		success,
+		done,
+		"smbc_readdirplus2 (20) failed\n");
 
 	/* Ensure the readdirplus2 and readdirplus names are the same. */
 	ret = strcmp(direntries_20->name, direntriesplus_20->name);
-	if (ret != 0) {
-		torture_fail_goto(tctx,
-			done,
-			talloc_asprintf(tctx,
+	torture_assert_goto(
+		tctx,
+		ret == 0,
+		success,
+		done,
+		talloc_asprintf(tctx,
 				"after seek (20) readdirplus2 name missmatch "
 				"file %s - got %s\n",
 				dirent_20->name,
 				direntries_20->name));
-	}
 
 	/* Ensure doing stat gets the same data. */
 	plus2_stat_path = talloc_asprintf(tctx,
 				"%s/%s",
 				dname,
 				direntriesplus_20->name);
-	if (plus2_stat_path == NULL) {
-		torture_fail_goto(tctx,
-			done,
-			"talloc fail\n");
-	}
+	torture_assert_not_null_goto(
+		tctx,
+		plus2_stat_path,
+		success,
+		done,
+		"talloc fail\n");
 
 	ret = smbc_stat(plus2_stat_path, &st);
 	torture_assert_int_equal_goto(tctx,
@@ -796,7 +816,7 @@ static bool torture_libsmbclient_readdirplus_seek(struct torture_context *tctx)
 
 static bool torture_libsmbclient_readdirplus2(struct torture_context *tctx)
 {
-	SMBCCTX *ctx;
+	SMBCCTX *ctx = NULL;
 	int dhandle = -1;
 	int fhandle = -1;
 	bool found = false;
@@ -1187,6 +1207,53 @@ out:
 	return ok;
 }
 
+static bool torture_libsmbclient_utimes(struct torture_context *tctx)
+{
+	const char *smburl = torture_setting_string(tctx, "smburl", NULL);
+	SMBCCTX *ctx = NULL;
+	struct stat st;
+	int fhandle, ret;
+	struct timeval tbuf[2];
+	bool ok;
+
+	if (smburl == NULL) {
+		torture_fail(tctx,
+			     "option --option=torture:smburl="
+			     "smb://user:password@server missing\n");
+	}
+
+	ok = torture_libsmbclient_init_context(tctx, &ctx);
+	torture_assert(tctx, ok, "Failed to init context");
+	smbc_set_context(ctx);
+
+	fhandle = smbc_open(smburl, O_RDWR|O_CREAT, 0644);
+	torture_assert_int_not_equal(tctx, fhandle, -1, "smbc_open failed");
+
+	ret = smbc_fstat(fhandle, &st);
+	torture_assert_int_not_equal(tctx, ret, -1, "smbc_fstat failed");
+
+	tbuf[0] = convert_timespec_to_timeval(get_atimespec(&st));
+	tbuf[1] = convert_timespec_to_timeval(get_mtimespec(&st));
+
+	tbuf[1] = timeval_add(&tbuf[1], 0, 100000); /* 100 msec */
+
+	ret = smbc_utimes(smburl, tbuf);
+	torture_assert_int_not_equal(tctx, ret, -1, "smbc_utimes failed");
+
+	ret = smbc_fstat(fhandle, &st);
+	torture_assert_int_not_equal(tctx, ret, -1, "smbc_fstat failed");
+
+	torture_assert_int_equal(
+		tctx,
+		get_mtimensec(&st) / 1000,
+		tbuf[1].tv_usec,
+		"smbc_utimes did not update msec");
+
+	smbc_close(fhandle);
+	smbc_unlink(smburl);
+	return true;
+}
+
 NTSTATUS torture_libsmbclient_init(TALLOC_CTX *ctx)
 {
 	struct torture_suite *suite;
@@ -1206,6 +1273,8 @@ NTSTATUS torture_libsmbclient_init(TALLOC_CTX *ctx)
 		torture_libsmbclient_readdirplus_seek);
 	torture_suite_add_simple_test(suite, "readdirplus2",
 		torture_libsmbclient_readdirplus2);
+	torture_suite_add_simple_test(
+		suite, "utimes", torture_libsmbclient_utimes);
 
 	suite->description = talloc_strdup(suite, "libsmbclient interface tests");
 
